#' Plot Transition and Trasnversion ratios.
#' @description Takes results generated from \code{titv} and plots the Ti/Tv ratios and contributions of 6 mutational conversion classes in each sample.
#'
#' @param res results generated by \code{\link{titv}}
#' @param plotType Can be 'bar', 'box' or 'both'. Defaults to 'both'
#' @param file basename for output file name. If given pdf will be generated.
#' @param width width of the plot, in inches.
#' @param height height of the plot, in inches.
#' @param color named vector of colors for each coversion class.
#' @param showBarcodes Whether to include sample names for barplot
#' @param textSize fontsize if showBarcodes is TRUE. Deafult 2.
#' @param baseFontSize font size. Deafult 12.
#' @param axisTextSize text size x and y tick labels. Default c(9,9).
#' @return None.
#' @seealso \code{\link{titv}}
#' @examples
#' laml.maf <- system.file("extdata", "tcga_laml.maf.gz", package = "maftools")
#' laml <- read.maf(maf = laml.maf)
#' laml.titv = titv(maf = laml, useSyn = TRUE)
#' plotTiTv(laml.titv)
#'
#' @export


plotTiTv = function(res = NULL, plotType = 'both', file = NULL, width = 6, height = 5,
                    color = NULL, showBarcodes = FALSE, textSize = 2, baseFontSize = 12, axisTextSize = c(9, 9)){

  if(is.null(color)){
    #col = RColorBrewer::brewer.pal(n = 6, name = 'Set2')
    col = c('coral4', 'lightcyan4', 'cornflowerblue', 'lightsalmon1', 'forestgreen', 'deeppink3')
    names(col) = c('C>T', 'C>G', 'C>A', 'T>A', 'T>C', 'T>G')
  }else{
    col = color
  }


  titv.frac = res$fraction.contribution
  titv.frac.melt = data.table::melt(data = titv.frac, id = 'Tumor_Sample_Barcode')
  conv.class = c('Ti', 'Ti', 'Tv', 'Tv', 'Tv', 'Tv')
  names(conv.class) = c("T>C", "C>T", "T>A", "T>G", "C>A", "C>G")
  titv.frac.melt$TiTv = conv.class[as.character(titv.frac.melt$variable)]
  # titv.frac.melt$variable = factor(x = titv.frac.melt$variable, levels = c('C-T', 'C-G', 'C-A', 'T-A', 'T-C', 'T-G'),
  #                                  labels = c('C>T', 'C>G', 'C>A', 'T>A', 'T>C', 'T>G'))

  # titv.frac.melt$TiTv = suppressWarnings( factor(x = titv.frac.melt$variable, levels = c('C>T', 'C>G', 'C>A', 'T>A', 'T>C', 'T>G'),
  #                                  labels = c('Ti', 'Tv', 'Tv', 'Tv', 'Ti', 'Tv')) )

  titv.contrib = suppressMessages(data.table::melt(res$TiTv.fractions, id = 'Tumor_Sample_Barcode'))
  titv.frac.melt$variable = factor(x = titv.frac.melt$variable,
                                   levels = c("T>C", "C>T", "T>A", "T>G", "C>A", "C>G"))

  titv_theme = cowplot::theme_cowplot(font_size = baseFontSize, line_size = 1)+cowplot::background_grid(major = 'xy', minor = 'none')+
    theme(axis.title.x = element_blank(), axis.text.x = element_text(face = "bold", size = axisTextSize[1]),
          axis.title.y = element_text(face = "bold"), axis.text.y = element_text(face = "bold", size = axisTextSize[2]),
          legend.title = element_blank(), legend.text = element_text(face = "bold"))


  p1 = ggplot(data = titv.frac.melt, aes(x = variable, y = value, fill = variable, color = variable)) +
    geom_boxplot(outlier.alpha = 0.6, outlier.size = 1, outlier.color = 'gray70') + ylim(0, 100)+
    ylab("% Mutations")+scale_fill_manual(values = col)+scale_color_manual(values = col)+
    titv_theme+theme(legend.position = 'none')+
    stat_summary(geom = "crossbar", width=0.65, fatten=0, color="black", fun.data = function(x){ return(c(y=median(x), ymin=median(x), ymax=median(x))) })
    #scale_color_manual(values = c('Ti' = 'maroon', 'Tv' = 'royalblue'))+cowplot::background_grid(major = 'xy', minor = 'none')

  p2 = ggplot(data = titv.contrib, aes(x = variable, y = value, fill = variable, color = variable))+
    geom_boxplot(outlier.alpha = 0.6, outlier.size = 1, fatten = NULL)+ylim(0, 100)+titv_theme+
    theme(legend.position = 'none', axis.title.y = element_blank())+
    scale_fill_manual(values = c('Ti' = '#969696', 'Tv' = '#737373'))+
    scale_color_manual(values = c('Ti' = '#969696', 'Tv' = '#737373'))+
    stat_summary(geom = "crossbar", width=0.65, fatten=0, color="black", fun.data = function(x){ return(c(y=median(x), ymin=median(x), ymax=median(x))) })

  top = suppressWarnings(cowplot::plot_grid(p1, p2, rel_widths = c(2,1)))

  titv.order = titv.frac.melt[,mean(value), by = .(variable)]
  titv.order = titv.order[order(V1, decreasing = TRUE)]
  orderlvl = as.character(titv.order$variable)
  titv.frac.melt$variable = factor(x = titv.frac.melt$variable, levels = orderlvl)

  p3 = ggplot(data = titv.frac.melt, aes(x = Tumor_Sample_Barcode, y = value))+geom_bar(aes(fill =variable) ,stat = 'identity', alpha = 0.8)+
    titv_theme+theme(legend.position = 'bottom', axis.ticks.x = element_blank(), axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1, size = textSize),
          axis.line.y = element_blank(), axis.line.x = element_blank())+
            ylab('% Mutations')+xlab('Samples')+scale_fill_manual(values = col)

  if(!showBarcodes){
    p3 = p3+theme(axis.text.x = element_blank())
  }

  p = suppressWarnings(cowplot::plot_grid(top, p3, nrow = 2))

  if(plotType == 'bar'){
    p.plot <- p3
  } else if(plotType == 'box'){
    p.plot <- top
  } else if(plotType == 'both'){
    p.plot <- p
  }else{
    stop('plotType can only be bar, box or both')
  }

  print(p.plot)

  if(!is.null(file)){
    cowplot::save_plot(filename = paste(file, 'pdf', sep='.'), plot = p.plot, base_height = height, base_width = width, bg = 'white')
  }
}
